from flask import Flask, render_template, request, redirect, url_for
import os
import re
import socket
import struct
import subprocess
import fcntl
import time

app = Flask(__name__)

# 获取网卡列表，过滤掉 lo 接口
def get_network_interfaces():
    return [iface for iface in os.listdir('/sys/class/net/') if iface != 'lo']

# 获取网卡的当前 IP 地址
def get_ip_address(ifname):
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        ip = fcntl.ioctl(
            s.fileno(),
            0x8915,  # SIOCGIFADDR
            struct.pack('256s', ifname[:15].encode('utf-8'))
        )
        return socket.inet_ntoa(ip[20:24])
    except IOError:
        return '未分配 IP'

# 读取网卡配置文件内容
def read_interface_config(interface):
    config_path = f'/etc/sysconfig/network-scripts/ifcfg-{interface}'
    if not os.path.exists(config_path):
        return {}
    
    config = {}
    with open(config_path, 'r') as file:
        for line in file:
            match = re.match(r'(\w+)=(.+)', line)
            if match:
                key, value = match.groups()
                config[key] = value.strip()
    return config

# 写入网卡配置文件
def write_interface_config(interface, config):
    config_path = f'/etc/sysconfig/network-scripts/ifcfg-{interface}'
    with open(config_path, 'w') as file:
        for key, value in config.items():
            file.write(f"{key}={value}\n")

# 计算网络地址
def calculate_network(ip, netmask):
    ip_parts = list(map(int, ip.split('.')))
    netmask_parts = list(map(int, netmask.split('.')))
    network_parts = [ip_parts[i] & netmask_parts[i] for i in range(4)]
    return f"{network_parts[0]}.{network_parts[1]}.{network_parts[2]}.{network_parts[3]}/{netmask_to_cidr(netmask)}"

# 转换子网掩码为 CIDR
def netmask_to_cidr(netmask):
    return sum([bin(int(x)).count('1') for x in netmask.split('.')])

def update_rt_tables(interface):
    rt_tables_path = '/etc/iproute2/rt_tables'
    
    try:
        with open(rt_tables_path, 'r') as file:
            table_entries = file.readlines()
        
        existing_numbers = set()
        existing_interfaces = set()
        
        for line in table_entries:
            if line.strip() and not line.startswith('#'):
                parts = line.split()
                if parts and parts[0].isdigit():
                    existing_numbers.add(int(parts[0]))
                    if len(parts) > 1:
                        existing_interfaces.add(parts[1])

        # 检查条目是否已经存在
        table_entry_name = f"rt_{interface}"
        if table_entry_name in existing_interfaces:
            return

        table_number = 1
        while table_number in existing_numbers:
            table_number += 1

        table_entry = f"{table_number} {table_entry_name}\n"

        with open(rt_tables_path, 'a') as file:
            file.write(table_entry)
    except Exception as e:
        print(f"更新 rt_tables 时出错: {e}")

# 重启指定网卡
def restart_interface(interface):
    try:
        subprocess.run(['ifdown', interface], check=True)
        subprocess.run(['ifup', interface], check=True)
        
        # 等待 15 秒
        time.sleep(15)
        
        subprocess.run(['sh', '/etc/rc.local'], check=False)
    except subprocess.CalledProcessError as e:
        print(f"重启接口 {interface} 时出错: {e}")


# 添加策略路由规则和默认路由，并将其加入到开机自动启动
def add_routing_rules(interface, ipaddr, netmask, gateway):
    network = calculate_network(ipaddr, netmask)
    
    # 更新 rc.local
    rc_local_path = '/etc/rc.local'
    
    with open(rc_local_path, 'r') as file:
        lines = file.readlines()
    
    # 移除已有的规则
    lines = [line for line in lines if f'rt_{interface}' not in line]
    
    # 添加新的规则
    lines.append(f'ip rule add from {network} table rt_{interface}\n')
    lines.append(f'ip route add default via {gateway} dev {interface} table rt_{interface}\n')
    
    with open(rc_local_path, 'w') as file:
        file.writelines(lines)

    os.chmod(rc_local_path, 0o755)

@app.route('/', methods=['GET', 'POST'])
def index():
    interfaces = get_network_interfaces()
    selected_ip = None

    if request.method == 'POST':
        interface = request.form['interface']
        bootproto = request.form['bootproto']

        config = read_interface_config(interface)
        config['BOOTPROTO'] = bootproto
        config['ONBOOT'] = 'yes'

        if bootproto == 'static':
            ipaddr = request.form['ipaddr']
            netmask = request.form['netmask']
            gateway = request.form['gateway']
            config['IPADDR'] = ipaddr
            config['NETMASK'] = netmask
            config['GATEWAY'] = gateway
            update_rt_tables(interface)
            add_routing_rules(interface, ipaddr, netmask, gateway)
        else:
            config.pop('IPADDR', None)
            config.pop('NETMASK', None)
            config.pop('GATEWAY', None)

        write_interface_config(interface, config)
        restart_interface(interface)

        return redirect(url_for('index'))

    if 'interface' in request.args:
        selected_interface = request.args.get('interface')
        selected_ip = get_ip_address(selected_interface)

    return render_template('index.html', interfaces=interfaces, selected_ip=selected_ip)

if __name__ == '__main__':
    app.run(host='0.0.0.0', port=5000)
